@@ -7,11 +7,10 @@ use Carp 'carp';
 use v5.10.1;
 
 our @CARP_NOT;
-our $VERSION = '0.15_1';
-
-use overload '""' => sub { $_[0]->to_string }, fallback => 1;
+our $VERSION = '0.17_1';
 
 our $DEFAULT_KEY_SIZE = 512;
+our $MAX_GEN_ROUNDS = 100;
 
 # Maximum number of tests for random prime generation = 100
 # Range of valid key sizes = 512 - 4096
@@ -24,7 +23,7 @@ our $DEFAULT_KEY_SIZE = 512;
 # TODO: Improve tests for _bitsize, b64url_encode, b64url_decode,
 #       hex2b64url, b64url2hex
 
-
+use overload '""' => sub { $_[0]->to_string }, fallback => 1;
 use Digest::SHA qw/sha256 sha256_hex/;
 use MIME::Base64 qw(decode_base64 encode_base64);
 
@@ -198,7 +197,9 @@ sub generate {
   my $psize = int( $size / 2 );
 
   my $n;
-  my $m = 100; # Maximum number of rounds
+
+  # Maximum number of rounds
+  my $m = $MAX_GEN_ROUNDS;
 
   my ($p, $q);
 
@@ -384,6 +385,7 @@ sub verify {
 };
 
 
+
 # Return MagicKey-String (public only)
 sub to_string {
   my $self = shift;
@@ -471,6 +473,7 @@ sub _verify_emsa_pkcs1_v1_5 {
 
   # The length of the signature is not
   # equivalent to the length of the RSA modulus
+  # TODO: This probably needs to check octetlength
   if (length($S) != $k) {
     carp 'Invalid signature';
     return;
@@ -491,7 +494,7 @@ sub _rsasp1 {
   # Key, message
   my ($K, $m) = @_;
 
-  if ($m >= $K->n) {
+  if ($m >= $K->n || $m < 0) {
     carp 'Message representative out of range';
     return;
   };
@@ -673,7 +676,7 @@ Crypt::MagicSignatures::Key - MagicKeys for the Salmon Protocol
 =head1 DESCRIPTION
 
 L<Crypt::MagicSignatures::Key> implements MagicKeys as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html>
+L<MagicSignatures Specification|https://cdn.rawgit.com/salmon-protocol/salmon-protocol/master/draft-panzer-magicsig-01.html>
 to sign messages of the L<Salmon Protocol|http://www.salmon-protocol.org/>.
 MagicSignatures is a "robust mechanism for digitally signing nearly arbitrary messages".
 See L<Crypt::MagicSignatures::Envelope> for using MagicKeys to sign MagicEnvelopes.
@@ -696,7 +699,7 @@ The MagicKey modulus.
   $mkey->e(3);
 
 The MagicKey public exponent.
-Defaults to C<65537>.
+Defaults to 65537.
 
 
 =head2 d
@@ -737,7 +740,7 @@ The MagicKey keysize in bits.
 
 
 The Constructor accepts MagicKeys in
-L<compact notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor13>
+L<compact notation|https://cdn.rawgit.com/salmon-protocol/salmon-protocol/master/draft-panzer-magicsig-01.html#anchor13>
 or by attributes.
 
 
@@ -751,6 +754,7 @@ Accepts the attributes C<size> and C<e>.
 In case no C<size> attribute is given, the default key size
 for generation is 512 bits, which is also the minimum size.
 The maximum size is 4096 bits.
+Random prime trials are limited to 100 rounds.
 
 
 =head2 sign
@@ -788,7 +792,7 @@ Returns a C<true> value on success and C<false> otherwise.
   my $priv_key = $mkey->to_string(1);
 
 Returns the public key as a string in
-L<compact notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor13>.
+L<compact notation|https://cdn.rawgit.com/salmon-protocol/salmon-protocol/master/draft-panzer-magicsig-01.html#anchor13>.
 If a C<true> value is passed to the method,
 the full key (including the private exponent if existing)
 is returned.
